Package: waveslim
Version: 1.8.5
Date: 2024-06-02
Title: Basic Wavelet Routines for One-, Two-, and Three-Dimensional
        Signal Processing
Author: Brandon Whitcher
Maintainer: Brandon Whitcher <bwhitcher@gmail.com>
Depends: R (>= 2.11.0), graphics, grDevices, stats, utils, multitaper
Suggests: fftw, covr
Description: Basic wavelet routines for time series (1D), image (2D) and array 
  (3D) analysis.  The code provided here is based on wavelet methodology 
  developed in Percival and Walden (2000); Gencay, Selcuk and Whitcher (2001); 
  the dual-tree complex wavelet transform (DTCWT) from Kingsbury (1999, 2001) as
  implemented by Selesnick; and Hilbert wavelet pairs (Selesnick 2001, 2002).  
  All figures in chapters 4-7 of GSW (2001) are reproducible using this package 
  and R code available at the book website(s) below.
License: BSD_3_clause + file LICENSE
URL: https://waveslim.blogspot.com
RoxygenNote: 7.3.1
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-06-03 18:31:59 UTC; brandon
Repository: CRAN
Date/Publication: 2024-06-04 13:20:03 UTC
