# waveslim

![Travis-CI Build Status](https://travis-ci.org/bjw34032/waveslim.svg?branch=master)
![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/waveslim)
![CRAN Downloads Badge](http://cranlogs.r-pkg.org/badges/grand-total/waveslim)
![CRAN Downloads Badge](http://cranlogs.r-pkg.org/badges/waveslim)

The R packages **waveslim** contains basic wavelet routines for time series (1D), image (2D) and array (3D) analysis. The code provided is based on wavelet methodology developed in 

* Percival and Walden (2000)
* Gencay, Selcuk and Whitcher (2001)
* the dual-tree complex wavelet transform (CWT) from Kingsbury (1999, 2001) as implemented by Selesnick, and 
* Hilbert wavelet pairs (Selesnick 2001, 2002)
