# waveslim

The R packages **waveslim** contains basic wavelet routines for time series (1D), image (2D) and array (3D) analysis. The code provided here is based on wavelet methodology developed in Percival and Walden (2000); Gencay, Selcuk and Whitcher (2001); the dual-tree complex wavelet transform (CWT) from Kingsbury (1999, 2001) as implemented by Selesnick; and Hilbert wavelet pairs (Selesnick 2001, 2002).

Travis-CI build for bjw34032/waveslim

[![Travis-CI Build Status](https://travis-ci.org/bjw34032/waveslim.svg?branch=master)](https://travis-ci.org/bjw34032/waveslim)

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/waveslim/)](https://CRAN.R-project.org/package=waveslim)
[![](http://cranlogs.r-pkg.org/badges/grand-total/waveslim)](https://CRAN.R-project.org/package=waveslim)
