% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cpi}
\alias{cpi}
\title{U.S. Consumer Price Index}
\format{
A vector containing 624 observations.
}
\source{
Unknown.
}
\usage{
data(cpi)
}
\description{
Monthly U.S. consumer price index from 1948:1 to 1999:12.
}
\references{
Gencay, R., F. Selcuk and B. Whitcher (2001) \emph{An
Introduction to Wavelets and Other Filtering Methods in Finance and
Economics}, Academic Press.
}
\keyword{datasets}
